#!C:\conda\envs\_build\python.exe
# EASY-INSTALL-ENTRY-SCRIPT: 'conda-build-all==0.13.3','console_scripts','conda-buildall'
__requires__ = 'conda-build-all==0.13.3'
import sys
from pkg_resources import load_entry_point

if __name__ == '__main__':
    sys.exit(
        load_entry_point('conda-build-all==0.13.3', 'console_scripts', 'conda-buildall')()
    )



import os
from contextlib import contextmanager
from collections import defaultdict

import conda.resolve
from conda.resolve import MatchSpec
import conda_build.config

import logging
from conda.resolve import stdoutlog, dotlog

conda_stdoutlog = stdoutlog

from conda.console import SysStdoutWriteHandler


NO_PACKAGES_EXCEPTION = tuple(getattr(conda.resolve, attr)
                              for attr in ['Unsatisfiable', 'NoPackagesFound'])

class StdoutNewline(SysStdoutWriteHandler):
    def emit(self, record):
        record.msg += '\n'
        SysStdoutWriteHandler.emit(self, record)


stdout = logging.getLogger('conda_build_all.version_matrix.stdoutlog')
stdout.addHandler(StdoutNewline())
stdout.setLevel(logging.WARNING)


@contextmanager
def override_conda_logging(level):
    # Override the conda logging handlers.

    # We need to import conda.fetch and conda.resolve to trigger the
    # creation of the loggers in the first place.
    import conda.fetch
    import conda.resolve

    levels = {}
    handlers = {}
    loggers = ['progress', 'progress.start', 'progress.update',
               'progress.stop', 'stdoutlog', 'stderrlog',
               'conda.resolve', 'dotupdate']

    for logger_name in loggers:
        logger = logging.getLogger(logger_name)
        levels[logger_name] = logger.level
        handlers[logger_name] = logger.handlers

        logger.setLevel(level)
        logger.handlers = []
    yield
    for logger_name in loggers:
        logger = logging.getLogger(logger_name)
        logger.setLevel(levels[logger_name])
        logger.handlers = handlers[logger_name]


def conda_special_versions(meta, index, version_matrix=None):
    """
    Returns a generator which configures conda build's PY and NPY versions
    according to the given version matrix. If no version matrix is given, it
    will be computed by :func:`special_case_version_matrix`.

    """
    if version_matrix is None:
        version_matrix = special_case_version_matrix(meta, index)

    for case in version_matrix:
        with setup_vn_mtx_case(case):
            yield case


def parse_specifications(requirements):
    """
    Parse a list of specifications, turning multi-line specifications into
    a single specification.
    """
    requirement_specs = defaultdict(list)
    # Generate a list of requirements for each spec name to ensure that
    # multi-line specs are handled.
    for spec in requirements:
        spec_details = spec.split(None, 1)
        if len(spec_details) == 2:
            # Package name and version spec were given, append the
            # version spec.
            requirement_specs[MatchSpec(spec).name].append(spec_details[1])
        elif spec_details[0] not in requirement_specs:
            # Only package name given (e.g. 'numpy'), and the package name is
            # not in the requirements yet, so add an empty list.
            requirement_specs[MatchSpec(spec).name] = []

    # Combine multi-line specs into a single line by assuming the requirements
    # should be and-ed.
    for spec_name, spec_list in requirement_specs.items():
        requirement_specs[spec_name] = ','.join(spec_list)

    # Turn these into MatchSpecs.
    requirement_specs = {name: MatchSpec(' '.join([name, spec]).strip())
                         for name, spec in requirement_specs.items()}

    return requirement_specs


def special_case_version_matrix(meta, index):
    """
    Return the non-orthogonal version matrix for special software within conda
    (numpy, python).

    For example, supposing a meta depended on numpy and python, and that there
    was a numpy 1.8 & 1.9 for python 2.7 but only a numpy 1.9 for python 3.5,
    the matrix should be:

        ([('python', '2.7'), ('numpy', '1.8')],
         [('python', '2.7'), ('numpy', '1.9')],
         [('python', '3.5'), ('numpy', '1.9')])

    Packages which don't depend on any of the special cases will return an
    iterable with an empty tuple. This is analogous to saying "a build is needed,
    but there are no special cases". Thus, code may reliably implement a loop such as:

    for case in special_case_version_matrix(...):
        ... setup the case ...
        ... build ...

    .. note::

        This algorithm does not deal with PERL and R versions at this time, and may be
        extended in the future to compute other special case dimensions (e.g. features).

    """
    r = conda.resolve.Resolve(index)

    requirements = meta.get_value('requirements/build', [])
    requirement_specs = parse_specifications(requirements)

    run_requirements = meta.get_value('requirements/run', [])
    run_requirement_specs = parse_specifications(run_requirements)

    # Thanks to https://github.com/conda/conda-build/pull/493 we no longer need to
    # compute the complex matrix for numpy versions unless a specific version has
    # been defined.
    np_spec = requirement_specs.get('numpy')
    np_run_spec = run_requirement_specs.get('numpy')
    if np_spec and np_run_spec and 'x.x' not in np_run_spec.spec:
        # A simple spec (just numpy) has been defined, so we can drop it from the
        # special cases.
        requirement_specs.pop('numpy')

    for pkg in requirement_specs:
        spec = requirement_specs[pkg]
        # We want to bake the version in, but we don't know what it is yet.
        if 'x.x' in spec.spec:
            # Remove the x.x part of the specification, assuming that if it
            # is present with other specifications they are and-ed together,
            # i.e. comma-separated.
            name, specification = spec.spec.split()
            spec_list = specification.split(',')
            no_xx = [s for s in spec_list if s != 'x.x']
            new_spec = ','.join(no_xx)
            if new_spec:
                ms = MatchSpec(' '.join([name, new_spec]))
            else:
                ms = MatchSpec(name)
            requirement_specs[pkg] = ms

    def minor_vn(version_str):
        """
        Take an string of the form 1.8.2, into string form 1.8
        """
        return '.'.join(version_str.split('.')[:2])

    cases = set()
    unsolvable_cases = set()

    def add_case_if_soluble(case):
        # Whilst we strictly don't need to, shortcutting cases we've already seen makes a
        # *huge* performance difference.
        if case in cases | unsolvable_cases:
            return

        specs = ([ms.spec for ms in requirement_specs.values()] +
                 ['{} {}*'.format(pkg, version) for pkg, version in case])

        try:
            # Figure out if this case is actually resolvable. We don't care how,
            # just that it could be.
            r.solve(specs)
        except NO_PACKAGES_EXCEPTION:
            unsolvable_cases.add(case)
        else:
            cases.add(case)

    with override_conda_logging(logging.WARN):
        if 'numpy' in requirement_specs:
            np_spec = requirement_specs.pop('numpy')
            py_spec = requirement_specs.pop('python', None)
            for numpy_pkg in r.get_pkgs(np_spec):
                np_vn = minor_vn(index[numpy_pkg.fn]['version'])
                numpy_deps = index[numpy_pkg.fn]['depends']
                numpy_deps = {MatchSpec(spec).name: MatchSpec(spec)
                              for spec in numpy_deps}
                # This would be problematic if python wasn't a dep of numpy.
                for python_pkg in r.get_pkgs(numpy_deps['python']):
                    if py_spec and not py_spec.match(python_pkg.fn):
                        continue
                    py_vn = minor_vn(index[python_pkg.fn]['version'])
                    case = (('python', py_vn),
                            ('numpy', np_vn),
                            )
                    add_case_if_soluble(case)
        elif 'python' in requirement_specs:
            py_spec = requirement_specs.pop('python')
            for python_pkg in r.get_pkgs(py_spec):
                py_vn = minor_vn(index[python_pkg.fn]['version'])
                case = (('python', py_vn), )
                add_case_if_soluble(case)

        if 'perl' in requirement_specs:
            pl_spec = requirement_specs.pop('perl')
            for case_base in list(cases or [()]):
                for perl_pkg in r.get_pkgs(pl_spec):
                    pl_vn = index[perl_pkg.fn]['version']
                    case = case_base + (('perl', pl_vn), )
                    add_case_if_soluble(case)
                if case_base in cases:
                    cases.remove(case_base)

        if 'r' in requirement_specs:
            r_spec = requirement_specs.pop('r')
            for case_base in list(cases or [()]):
                for r_pkg in r.get_pkgs(r_spec):
                    r_vn = index[r_pkg.fn]['version']
                    case = case_base + (('r', r_vn), )
                    add_case_if_soluble(case)
                if case_base in cases:
                    cases.remove(case_base)

    # Deal with the fact that building a Python recipe itself requires a special case
    # version. This comes down to the odd decision in
    # https://github.com/conda/conda-build/commit/3dddeaf3cf5e85369e28c8f96e24c2dd655e36f0.
    if meta.name() == 'python' and not cases:
        cases.add((('python', '.'.join(meta.version().split('.', 2)[:2])),))

    # Put an empty case in to allow simple iteration of the results.
    if not cases:
        cases.add(())

    return set(cases)


def filter_cases(cases, extra_specs):
    """
    cases might look like:

        cases = ([('python', '2.7'), ('numpy', '1.8')],
                 [('python', '2.7'), ('numpy', '1.9')],
                 [('python', '3.5'), ('numpy', '1.8')],
                 )

    Typically extra_specs comes from the environment specification.

    """
    specs = [MatchSpec(spec) for spec in extra_specs]

    for case in cases:
        # Invent a sensible "tar.bz2" name which we can use to invoke conda's
        # MatchSpec matching.
        cases_by_pkg_name = {name: '{}-{}.0-0.tar.bz2'.format(name, version)
                             for name, version in case}
        match = []
        for spec in specs:
            # Only run the filter on the packages in cases.
            if spec.name in cases_by_pkg_name:
                match.append(bool(spec.match(cases_by_pkg_name[spec.name])))
        if all(match):
            yield case


def keep_top_n_major_versions(cases, n=2):
    """
    Remove all but the top n major version cases for each package in cases.

    Parameters
    ----------
    cases
        The cases to filter. See filter_cases for a definition of cases.
    n : integer >= 0
        The number of major versions to keep. Default is ``2``. 0 results in all
        major versions being kept.

    """
    name_to_major_versions = {}
    for case in cases:
        for name, version in case:
            name_to_major_versions.setdefault(name, set()).add(int(version.split('.')[0]))
    cutoff = {name: sorted(majors)[-n:] for name, majors in name_to_major_versions.items()}
    for case in cases:
        keeper = True
        for name, version in case:
            if int(version.split('.')[0]) not in cutoff[name]:
                keeper = False
        if keeper:
            yield case


def keep_top_n_minor_versions(cases, n=2):
    """
    Remove all but the top n minor version cases for each package in cases.
    This will not do any major version filtering, so two major versions with
    many minor versions will result in n x 2 cases returned.

    Parameters
    ----------
    cases
        The cases to filter. See filter_cases for a definition of cases.
    n : integer >= 0
        The number of minor versions to keep. Default is ``2``. 0 results in all
        minor versions being kept.

    """
    mapping = {}
    for case in cases:
        for name, version in case:
            major = int(version.split('.')[0])
            minor = int(version.split('.')[1])
            mapping.setdefault((name, major), set()).add(minor)
    cutoff = {key: sorted(minors)[-n:] for key, minors in mapping.items()}
    for case in cases:
        keeper = True
        for name, version in case:
            major = int(version.split('.')[0])
            minor = int(version.split('.')[1])
            if minor not in cutoff[(name, major)]:
                keeper = False
        if keeper:
            yield case




import os


def resolve_dependencies(package_dependencies):
    """
    Given a dictionary mapping a package to its dependencies, return a
    generator of packages to install, sorted by the required install
    order.

    >>> deps = resolve_dependencies({'a': ['b', 'c'], 'b': ['c'],
                                     'c': ['d'], 'd': []})
    >>> list(deps)
    ['d', 'c', 'b', 'a']

    """
    remaining_dependencies = package_dependencies.copy()
    completed_packages = []

    # A maximum of 10000 iterations. Beyond that and there is probably a
    # problem.
    for failsafe in range(10000):
        for package, deps in sorted(remaining_dependencies.copy().items()):
            if all(dependency in completed_packages for dependency in deps):
                completed_packages.append(package)
                remaining_dependencies.pop(package)
                yield package
            else:
                # Put a check in to ensure that all the dependencies were
                # defined as packages, otherwise we will never succeed.
                for dependency in deps:
                    if dependency not in package_dependencies:
                        msg = ('The package {} depends on {}, but it was not '
                               'part of the package_dependencies dictionary.'
                               ''.format(package, dependency))
                        raise ValueError(msg)

        # Close off the loop if we've completed the dependencies.
        if not remaining_dependencies:
            break
    else:
        raise ValueError('Dependencies could not be resolved. '
                         'Remaining dependencies: {}'
                         ''.format(remaining_dependencies))



"""
Build all the conda recipes in the given directory sequentially if they do not
already exist on the given binstar channel.
Building is done in order of dependencies (circular dependencies are not supported).
Once a build is complete, the distribution will be uploaded (provided BINSTAR_TOKEN is
defined), and the next package will be processed.

"""
from __future__ import print_function

from copy import deepcopy
import glob
import logging
import mock
import os
import subprocess
from argparse import Namespace

from binstar_client.utils import get_binstar
import binstar_client
from conda.api import get_index
import conda.config
import conda_build.config
from conda_build.metadata import MetaData
from conda_build.build import bldpkg_path
import conda.config

from . import order_deps
from . import build
from . import inspect_binstar
from . import version_matrix as vn_matrix
from . import resolved_distribution


def package_built_name(package, root_dir):
    package_dir = os.path.join(root_dir, package)
    meta = MetaData(package_dir)
    return bldpkg_path(meta)


def distribution_exists(binstar_cli, owner, metadata):
    fname = '{}/{}.tar.bz2'.format(conda.config.subdir, metadata.dist())
    try:
        r = binstar_cli.distribution(owner, metadata.name(), metadata.version(),
                                     fname)
        exists = True
    except binstar_client.errors.NotFound:
        exists = False
    return exists


def list_metas(directory, max_depth=0):
    """
    Get the build metadata of all recipes in a directory.

    The order of metas from this function is not guaranteed.

    Parameters
    ----------
    directory
        Where to start looking for metas using os.walk.
    max_depth : int
        How deep to recurse when looking for recipes.
        A value ``<=0`` will recurse indefinitely. A value of 1
        will look in the given directory for a meta.yaml.
        (default: 0)

    """
    packages = []
    current_depth = max_depth
    root = os.path.normpath(directory)
    for new_root, dirs, files in os.walk(root, followlinks=True):
        depth = new_root[len(root):].count(os.path.sep) + 1
        if max_depth > 0 and depth >= max_depth:
            del dirs[:]

        if 'meta.yaml' in files:
            packages.append(MetaData(new_root))
    return packages


def sort_dependency_order(metas):
    """Sort the metas into the order that they must be built."""
    meta_named_deps = {}
    buildable = [meta.name() for meta in metas]
    for meta in metas:
        meta = deepcopy(meta)

        # In order to deal with selectors impacting sort order, we completely
        # ignore them for the sake of ordering. This decision was taken in the
        # light of https://github.com/SciTools/conda-build-all/issues/30 as a
        # pragmatic performance choice.
        def select_lines(data, *args, **kwargs):
            # Just return the data without removing any of the lines. This
            # is only a suitable solution when selectors are also comments.
            return data

        with mock.patch('conda_build.metadata.select_lines', new=select_lines):
            with mock.patch('conda_build.jinja_context.select_lines', new=select_lines):
                meta.parse_again(permit_undefined_jinja=True)

        # Now that we have re-parsed the metadata with selectors unconditionally
        # included, we can get the run and build dependencies and do a toposort.
        all_deps = ((meta.get_value('requirements/run', []) or []) +
                    (meta.get_value('requirements/build', []) or []))
        # Remove version information from the name.
        all_deps = [dep.split(' ', 1)[0] for dep in all_deps]
        meta_named_deps[meta.name()] = [dep for dep in all_deps if dep in buildable]
    sorted_names = list(order_deps.resolve_dependencies(meta_named_deps))
    return sorted(metas, key=lambda meta: sorted_names.index(meta.name()))


class Builder(object):
    def __init__(self, conda_recipes_directory,
                 inspection_channels, inspection_directories,
                 artefact_destinations,
                 matrix_conditions, matrix_max_n_major_minor_versions=(2, 2)):
        """
        Build a directory of conda recipes sequentially, if they don't already exist in the inspection locations.

        Parameters
        ----------
        conda_recipes_directory : string
            The path to the directory in which to look for conda recipes.
        inspection_channels : iterable
            The conda channels to inspect to determine whether a recipe has already been built.
        inspection_directories : iterable
            The local directories to inspect to determine whether a recipe has already been built.
        artefact_destinations : iterable of conda_build_all.artefact_destination.ArtefactDestination
            The destinations for the built artefact to go to.
        matrix_conditions : iterable of conda specifications
            The conditions to apply when determining whether a recipe should be built
        matrix_max_n_major_minor_versions : pair of ints
            The number of major and minor versions to preserve for each resolved recipe. For instance,
            if a recipe can be built against np 1.7, 1.8 and 1.9, and the number of minor versions is 2,
            the build matrix will prune the 1.7 option.

        """
        self.conda_recipes_directory = conda_recipes_directory
        self.inspection_channels = inspection_channels or []
        self.inspection_directories = inspection_directories
        self.artefact_destinations = artefact_destinations
        self.matrix_conditions = matrix_conditions
        self.matrix_max_n_major_minor_versions = matrix_max_n_major_minor_versions

    def fetch_all_metas(self):
        """
        Return the conda recipe metas, in the order they should be built.

        """
        conda_recipes_directory = os.path.abspath(os.path.expanduser(self.conda_recipes_directory))
        recipe_metas = list_metas(conda_recipes_directory)
        recipe_metas = sort_dependency_order(recipe_metas)
        return recipe_metas

    def find_existing_built_dists(self, recipe_metas):
        recipes = tuple([meta, None] for meta in recipe_metas)
        if self.inspection_channels:
            # For an unknown reason we are unable to cache the get_index call. There is a
            # test which fails against v3.18.6 if use_cache is True.
            index = get_index(self.inspection_channels, prepend=False, use_cache=False)
            # We look to see if a distribution exists in the channel. Note: This is not checking
            # there is a distribution for this platform. This isn't a big deal, as channels are
            # typically split by platform. If this changes, we would need to re-consider how this
            # is implemented.

            # We temporarily workaround the index containing the channel information in the key.
            # We should deal with this properly though.
            index = {meta['fn']: meta for meta in index.values()}

            for recipe_pair in recipes:
                meta, dist_location = recipe_pair
                if meta.pkg_fn() in index:
                    recipe_pair[1] = index[meta.pkg_fn()]['channel']
        if self.inspection_directories:
            for directory in self.inspection_directories:
                files = glob.glob(os.path.join(directory, '*.tar.bz2'))
                fnames = [os.path.basename(fpath) for fpath in files]
                for recipe_pair in recipes:
                    meta, dist_location = recipe_pair
                    if dist_location is None and meta.pkg_fn() in fnames:
                        recipe_pair[1] = directory
        return recipes

    def build(self, meta):
        print('Building ', meta.dist())
        with meta.vn_context():
            return bldpkg_path(build.build(meta.meta))

    def compute_build_distros(self, index, recipes):
        """
        Given the recipes which are to be built, return a list of BakedDistribution instances
        for all distributions that should be built.

        """
        all_distros = []
        index = index.copy()
        for meta in recipes:
            distros = resolved_distribution.ResolvedDistribution.resolve_all(meta, index,
                                                                             self.matrix_conditions)
            cases = [distro.special_versions for distro in distros]
            cases = list(vn_matrix.keep_top_n_major_versions(cases, n=self.matrix_max_n_major_minor_versions[0]))
            cases = list(vn_matrix.keep_top_n_minor_versions(cases, n=self.matrix_max_n_major_minor_versions[1]))
            for distro in distros:
                if distro.special_versions in cases:
                    # Update the index with this distribution so that it can be considered by the version matrix.
                    if distro.pkg_fn() not in index:
                        index[distro.pkg_fn()] = distro.info_index()
                    all_distros.append(distro)

        return all_distros

    def main(self):
        index = get_index(use_cache=True)

        # If it is not already defined with environment variables, we set the CONDA_NPY
        # to the latest possible value. Since we compute a build matrix anyway, this is 
        # useful to prevent conda-build bailing if the recipe depends on it (e.g.
        # ``numpy x.x``), and to ensure that recipes that don't care which version they want
        # at build/test time get a sensible version.
        if conda_build.config.config.CONDA_NPY is None:
            resolver = conda.resolve.Resolve(index)
            npy = resolver.get_pkgs('numpy', emptyok=True)
            if npy:
                version = ''.join(max(npy).version.split('.')[:2])
                conda_build.config.config.CONDA_NPY = version

        recipe_metas = self.fetch_all_metas()
        print('Resolving distributions from {} recipes... '.format(len(recipe_metas)))

        all_distros = self.compute_build_distros(index, recipe_metas)
        print('Computed that there are {} distributions from the {} '
              'recipes:'.format(len(all_distros), len(recipe_metas)))
        recipes_and_dist_locn = self.find_existing_built_dists(all_distros)

        print('Resolved dependencies, will be built in the following order: \n\t{}'.format(
              '\n\t'.join(['{} (will be built: {})'.format(meta.dist(), dist_locn is None)
                           for meta, dist_locn in recipes_and_dist_locn])))

        for meta, built_dist_location in recipes_and_dist_locn:
            was_built = built_dist_location is None
            if was_built:
                built_dist_location = self.build(meta)
            self.post_build(meta, built_dist_location, was_built)

    def post_build(self, meta, built_dist_location, was_built):
        """
        The post build phase occurs whether or not a build has actually taken place.
        It is the point at which a distribution is transfered to the desired artefact
        location.

        Parameters
        ----------
        meta : MetaData
            The distribution for which we are running the post-build phase
        build_dist_location : str
            The location of the built .tar.bz2 file for the given meta.

        """
        for artefact_destination in self.artefact_destinations:
            artefact_destination.make_available(meta, built_dist_location, was_built)



import unittest

import conda.config
from conda.resolve import MatchSpec

from conda_build_all.version_matrix import (parse_specifications,
                                            special_case_version_matrix,
                                            filter_cases,
                                            keep_top_n_major_versions,
                                            keep_top_n_minor_versions)
from conda_build_all.tests.unit.dummy_index import DummyPackage, DummyIndex


class Test_special_case_version_matrix(unittest.TestCase):
    def setUp(self):
        self.pkgs = {'a': DummyPackage('pkgA', ['python', 'numpy']),
                     'b': DummyPackage('b', ['c']),
                     'c': DummyPackage('c'),
                     'b_alt': DummyPackage('b', ['c', 'd']),
                     'd': DummyPackage('d')}
        self.index = DummyIndex()

    def test_no_case(self):
        # No cases should still give us a result with a single case in it.
        a = DummyPackage('pkgA', ['wibble'])
        self.index.add_pkg('python', '2.7.2')
        self.index.add_pkg('wibble', '3.5.0')
        r = special_case_version_matrix(a, self.index)
        self.assertEqual(r, set([()]))

    def test_python_itself(self):
        a = DummyPackage('python', version="a.b.c")
        r = special_case_version_matrix(a, self.index)
        self.assertEqual(r, set(((('python', 'a.b'),),
                                ))
                         )

    def test_python(self):
        a = DummyPackage('pkgA', ['python'])
        self.index.add_pkg('python', '2.7.2')
        self.index.add_pkg('python', '3.5.0')
        r = special_case_version_matrix(a, self.index)
        self.assertEqual(r, set(((('python', '2.7'),),
                                 (('python', '3.5'),),
                                ))
                         )

    def test_constrained_python(self):
        a = DummyPackage('pkgA', ['python <3'])
        self.index.add_pkg('python', '2.7.2')
        self.index.add_pkg('python', '3.5.0')
        r = special_case_version_matrix(a, self.index)
        self.assertEqual(r, set(((('python', '2.7'),
                                  ),
                                ))
                         )

    def test_numpy_simplest_case(self):
        a = DummyPackage('pkgA', ['python', 'numpy'])
        self.index.add_pkg('numpy', '1.8.0', 'py27', depends=['python'])
        self.index.add_pkg('python', '2.7.2')
        r = special_case_version_matrix(a, self.index)
        self.assertEqual(r, set([(('python', '2.7'), ('numpy', '1.8')),
                                ])
                         )

    def test_numpy_without_python(self):
        # Conda recipes which do not depend on python, but do on python, do
        # not have the full conda metadata, but still need to be handled.
        a = DummyPackage('pkgA', ['numpy'])
        self.index.add_pkg('numpy', '1.8.0', 'py27', depends=['python'])
        self.index.add_pkg('python', '2.7.2')
        r = special_case_version_matrix(a, self.index)
        self.assertEqual(r, set([(('python', '2.7'), ('numpy', '1.8')),
                                ])
                         )

    def test_numpy_repeated_python27(self):
        # Repeating python 2.7 will result in the latest version being found
        a = DummyPackage('pkgA', ['python', 'numpy'])
        self.index.add_pkg('numpy', '1.8.0', 'py27', depends=['python <3'])
        self.index.add_pkg('python', '2.7.2')
        self.index.add_pkg('python', '2.7.0')
        r = special_case_version_matrix(a, self.index)
        self.assertEqual(r, set([(('python', '2.7'), ('numpy', '1.8')),
                                ])
                         )

    def test_numpy_repeated_python(self):
        a = DummyPackage('pkgA', ['python', 'numpy'])
        self.index.add_pkg('numpy', '1.8.0', 'py27', depends=['python <3'])
        self.index.add_pkg('numpy', '1.8.0', 'py35', depends=['python'])
        self.index.add_pkg('numpy', '1.9.0', 'py35', depends=['python >=3'])
        self.index.add_pkg('python', '2.7.2')
        self.index.add_pkg('python', '3.5.0')
        r = special_case_version_matrix(a, self.index)
        self.assertEqual(r, set(((('python', '2.7'), ('numpy', '1.8')),
                                 (('python', '3.5'), ('numpy', '1.8')),
                                 (('python', '3.5'), ('numpy', '1.9')),
                                ))
                         )

    def test_dependency_on_py27(self):
        # If a dependency can't hit the python version, it should not
        # be considered a case.
        a = DummyPackage('pkgA', ['python', 'oldschool'])
        self.index.add_pkg('oldschool', '1.8.0', 'py27', depends=['python <3'])
        self.index.add_pkg('python', '2.7.2')
        self.index.add_pkg('python', '3.5.0')
        r = special_case_version_matrix(a, self.index)
        # No python 3 should be here.
        self.assertEqual(r, set([(('python', '2.7'),
                                   ),
                                ]
                               ))

    def construct_numpy_index(self, python_versions, numpy_versions):
        """
        Set up an index with several versions of python and numpy.
        """
        for python_version in python_versions:
            python_build_string = 'py' + python_version.replace('.', '')
            self.index.add_pkg('python', python_version)
            for numpy_version in numpy_versions:
                # Add a patch version to each numpy since that is how the
                # versions are numbered.
                self.index.add_pkg('numpy',
                                   numpy_version + '.2',
                                   python_build_string,
                                   depends=['python ' + python_version])

    def test_numpy_xx_only(self):
        # Only a numpy x.x spec.

        # Build an index that contains numpy 1.9 and 1.10 on python 2.7 and
        # 3.5 for a total of 4 numpy/python combinations.
        pythons = ['2.7', '3.5']
        # Only major/minor in the numpy list here because that is what is used
        # for the build matrix.
        numpys = ['1.9', '1.10']
        self.construct_numpy_index(pythons, numpys)

        # Case 1: Only a numpy x.x spec...

        numpy_dep_case = ('numpy x.x', 'python')

        # ...expect all four cases to be in the matrix.
        expect_result = []
        for python in pythons:
            for numpy in numpys:
                expect_result.append((('python', python), ('numpy', numpy)))

        a = DummyPackage('pkgA', numpy_dep_case, numpy_dep_case)

        r = special_case_version_matrix(a, self.index)
        self.assertEqual(set(r), set(expect_result),
                         msg='got: {}\nexpected: {}'.format(r, expect_result))

    def test_numpy_xx_and_nonrestrictive_specifciation(self):
        # Case 2:
        #   A numpy x.x spec and a numpy version restriction which does NOT
        #   exclude any of the cases in the DummyIndex.

        # Build an index that contains numpy 1.9 and 1.10 on python 2.7 and
        # 3.5 for a total of 4 numpy/python combinations.
        pythons = ['2.7', '3.5']
        # Only major/minor in the numpy list here because that is what is used
        # for the build matrix.
        numpys = ['1.9', '1.10']
        self.construct_numpy_index(pythons, numpys)

        # A numpy x.x spec and a numpy version restriction which does NOT
        # exclude any of the cases in the DummyIndex.
        numpy_dep_case = ('numpy x.x', 'numpy >1.6', 'python')

        # As in case 1, expect all four cases to be in the matrix.
        expect_result = []
        for python in pythons:
            for numpy in numpys:
                expect_result.append((('python', python), ('numpy', numpy)))

        a = DummyPackage('pkgA', numpy_dep_case, numpy_dep_case)

        r = special_case_version_matrix(a, self.index)
        self.assertEqual(set(r), set(expect_result),
                         msg='got: {}\nexpected: {}'.format(r, expect_result))

    def test_numpy_xx_and_restrictive_specifcation(self):
        # Case 3:
        #   A numpy x.x spec and a numpy version restriction which does
        #   eliminate one the numpy versions in the DummyIndex.

        # Build an index that contains numpy 1.9 and 1.10 on python 2.7 and
        # 3.5 for a total of 4 numpy/python combinations.
        pythons = ['2.7', '3.5']
        # Only major/minor in the numpy list here because that is what is used
        # for the build matrix.
        numpys = ['1.9', '1.10']
        self.construct_numpy_index(pythons, numpys)

        # A numpy x.x spec and a numpy version restriction which does
        # eliminate one the numpy versions in the DummyIndex.
        numpy_dep_case = ('numpy x.x', 'numpy >=1.10', 'python')

        # Expect only the numpy 1.9 case to survive.
        expect_result = []
        for python in pythons:
            for numpy in numpys[1:]:
                expect_result.append((('python', python), ('numpy', numpy)))

        a = DummyPackage('pkgA', numpy_dep_case, numpy_dep_case)

        r = special_case_version_matrix(a, self.index)
        self.assertEqual(set(r), set(expect_result),
                         msg='got: {}\nexpected: {}'.format(r, expect_result))

    def test_perl_matrix(self):
        a = DummyPackage('pkgA', ['perl'])
        self.index.add_pkg('perl', '4.5.6')
        self.index.add_pkg('perl', '4.5.7')
        r = special_case_version_matrix(a, self.index)
        self.assertEqual(r, set(((('perl', '4.5.6'),),
                                 (('perl', '4.5.7'),),
                                ))
                         )

    def test_perl_and_python_matrix(self):
        a = DummyPackage('pkgA', ['perl', 'python'])
        self.index.add_pkg('perl', '4.5.6')
        self.index.add_pkg('perl', '4.5.7')
        self.index.add_pkg('python', '2.7')
        self.index.add_pkg('python', '3.5')

        r = special_case_version_matrix(a, self.index)
        expected = set(((('python', '3.5'), ('perl', '4.5.7')),
                        (('python', '2.7'), ('perl', '4.5.7')),
                        (('python', '2.7'), ('perl', '4.5.6')),
                        (('python', '3.5'), ('perl', '4.5.6'))))
        self.assertEqual(r, expected)

    def test_r_matrix(self):
        a = DummyPackage('pkgA', ['r'])
        self.index.add_pkg('r', '4.5.6')
        self.index.add_pkg('r', '4.5.7')
        r = special_case_version_matrix(a, self.index)
        self.assertEqual(r, set(((('r', '4.5.6'),),
                                 (('r', '4.5.7'),),
                                ))
                         )

    def test_r_and_py_and_perl_matrix(self):
        a = DummyPackage('pkgA', ['perl', 'python', 'r'])
        self.index.add_pkg('perl', '4.5.6')
        self.index.add_pkg('perl', '4.5.7')
        self.index.add_pkg('python', '2.7')
        self.index.add_pkg('python', '3.5')
        self.index.add_pkg('r', '1.2.3')
        self.index.add_pkg('r', '4.5.6')

        r = special_case_version_matrix(a, self.index)
        expected = set(((('python', '2.7'), ('perl', '4.5.6'), ('r', '1.2.3')),
                        (('python', '2.7'), ('perl', '4.5.6'), ('r', '4.5.6')),
                        (('python', '3.5'), ('perl', '4.5.6'), ('r', '1.2.3')),
                        (('python', '3.5'), ('perl', '4.5.7'), ('r', '1.2.3')),
                        (('python', '3.5'), ('perl', '4.5.7'), ('r', '4.5.6')),
                        (('python', '2.7'), ('perl', '4.5.7'), ('r', '4.5.6')),
                        (('python', '2.7'), ('perl', '4.5.7'), ('r', '1.2.3')),
                        (('python', '3.5'), ('perl', '4.5.6'), ('r', '4.5.6')),
                        ))
        self.assertEqual(r, expected)


class Test_parse_specification(unittest.TestCase):
    def test_specification_no_duplicates(self):
        # Do specifications that are all on one-liners get handled correctly?
        input_spec = ['numpy', 'scipy', 'python']
        expected_match_spec = [MatchSpec(spec) for spec in input_spec]
        expected_output = {ms.name: ms for ms in expected_match_spec}

        output_spec = parse_specifications(input_spec)
        self.assertEqual(expected_output, output_spec)

    def test_specification_duplicates_with_version(self):
        # If there are duplicates lines in the specifications, and each
        # contains a non-trivial version specification, do they get combined
        # as expected?
        input_spec = ['numpy >=1.7', 'numpy <1.10', 'python']
        expected_match_spec = [MatchSpec(spec) for spec in ['numpy >=1.7,<1.10', 'python']]
        expected_output = {ms.name: ms for ms in expected_match_spec}
        output_spec = parse_specifications(input_spec)
        self.assertEqual(expected_output, output_spec)

    def test_three_part_spec_preserved(self):
        # A conda specification may contain up to three parts. Make sure those
        # are preserved.

        input_spec = ['numpy 1.8.1 py27_0', 'python']
        expected_match_spec = [MatchSpec(spec) for spec in input_spec]
        expected_output = {ms.name: ms for ms in expected_match_spec}
        output_spec = parse_specifications(input_spec)
        self.assertEqual(expected_output, output_spec)

    def test_multiline_spec_with_one_three_part_spec(self):
        # The expected output here is to have the specifications combined
        # with a comma even though the result is not a valid conda version
        # specification. However, the original multi-line version is not
        # valid either.

        input_spec = ['numpy 1.8.1 py27_0', 'numpy 1.8*', 'python']
        expected_match_spec = [MatchSpec(spec) for spec
                               in ['numpy 1.8.1 py27_0,1.8*', 'python']]
        expected_output = {ms.name: ms for ms in expected_match_spec}
        output_spec = parse_specifications(input_spec)
        self.assertEqual(expected_output, output_spec)

    def test_specification_with_blank(self):
        # Does a multiline specification, one of which is just the package
        # name, properly combine the other specifications on the other lines?

        input_spec = ('numpy 1.9', 'numpy', 'numpy <1.11', 'python 2.7')

        expected_match_spec = [MatchSpec(spec) for spec
                               in ['numpy 1.9,<1.11', 'python 2.7']]
        expected_output = {ms.name: ms for ms in expected_match_spec}
        output_spec = parse_specifications(input_spec)
        self.assertEqual(expected_output, output_spec)


class CasesTestCase(unittest.TestCase):
    def setUp(self):
        self.item = {
                     'py26': ('python', '2.6'),
                     'py27': ('python', '2.7'),
                     'py34': ('python', '3.4'),
                     'py35': ('python', '3.5'),
                     'o12': ('other', '1.2'),
                     'o13': ('other', '1.3'),
                     'np19': ('numpy', '1.9'),
                     'np110': ('numpy', '1.10'),
                     'np21': ('numpy', '2.1'),
                     }


class Test_filter_cases(CasesTestCase):
    # n.b. We should be careful not to test MatchSpec functionality here.

    def test_nothing(self):
        self.assertEqual(list(filter_cases([], [])), [])

    def test_no_filter(self):
        cases = ([self.item['py26']],
                 [self.item['py35']])
        self.assertEqual(tuple(filter_cases(cases, [])), cases)

    def test_single_filter(self):
        cases = ([self.item['py26']],
                 [self.item['py35']])
        self.assertEqual(tuple(filter_cases(cases, ['python >=3'])), cases[1:])

    def test_multiple_filter(self):
        cases = ([self.item['py26']],
                 [self.item['py34']],
                 [self.item['py35']])
        self.assertEqual(tuple(filter_cases(cases, ['python >=3', 'python <=3.4'])), cases[1:2])

    def test_multiple_filter_with_numpy(self):
        cases = ([self.item['py26'], self.item['np110']],
                 [self.item['py34'], self.item['np19']],
                 [self.item['py35'], self.item['np110']])
        self.assertEqual(tuple(filter_cases(cases, ['python >=3', 'numpy 1.10.*'])), cases[2:])

    def test_other_cases(self):
        cases = ([self.item['py26'], self.item['o12']],
                 [self.item['py34'], self.item['o12']],
                 [self.item['py35'], self.item['o13']])
        self.assertEqual(tuple(filter_cases(cases, ['other 1.2.*'])), cases[:2])


class Test_keep_top_n_major_versions(CasesTestCase):
    def test_keep_less_than_n(self):
        cases = ([self.item['py26']],)
        self.assertEqual(tuple(keep_top_n_major_versions(cases, 2)),
                         cases)

    def test_keep_1(self):
        cases = ([self.item['py27']],
                 [self.item['py35']])
        self.assertEqual(tuple(keep_top_n_major_versions(cases, 1)),
                         cases[1:])

    def test_keep_2(self):
        cases = ([self.item['py27']],
                 [self.item['py35']])
        self.assertEqual(tuple(keep_top_n_major_versions(cases, 2)),
                         cases)

    def test_keep_0(self):
        cases = ([self.item['py27']],
                 [self.item['py35']])
        self.assertEqual(tuple(keep_top_n_major_versions(cases, 0)),
                         cases)

    def test_multiple_packages(self):
        cases = ([self.item['py35'], self.item['np110']],
                 [self.item['py35'], self.item['np21']],
                 [self.item['py27'], self.item['np110']],
                 [self.item['py27'], self.item['np21']])
        self.assertEqual(tuple(keep_top_n_major_versions(cases, 1)),
                         cases[1:2])

    def test_multiple_packages_leaves_nothing(self):
        cases = ([self.item['py35'], self.item['np110']],
                 [self.item['py27'], self.item['np21']])
        self.assertEqual(tuple(keep_top_n_major_versions(cases, 1)),
                         ())


class Test_keep_top_n_minor_versions(CasesTestCase):
    def test_keep_less_than_n(self):
        cases = ([self.item['py26']],)
        self.assertEqual(tuple(keep_top_n_minor_versions(cases, 2)),
                         cases)

    def test_keep_1(self):
        cases = ([self.item['py26']],
                 [self.item['py27']],
                 [self.item['py34']],
                 [self.item['py35']])
        self.assertEqual(tuple(keep_top_n_minor_versions(cases, 1)),
                         cases[1::2])

    def test_keep_2(self):
        cases = ([self.item['py26']],
                 [self.item['py27']],
                 [self.item['py35']])
        self.assertEqual(tuple(keep_top_n_minor_versions(cases, 2)),
                         cases)

    def test_keep_0(self):
        cases = ([self.item['py26']],
                 [self.item['py35']])
        self.assertEqual(tuple(keep_top_n_minor_versions(cases, 0)),
                         cases)

    def test_multiple_packages(self):
        cases = ([self.item['py26'], self.item['np19']],
                 [self.item['py26'], self.item['np110']],
                 [self.item['py27'], self.item['np110']])
        self.assertEqual(tuple(keep_top_n_minor_versions(cases, 1)),
                         cases[2:])



#    def test_keep_0(self):



if __name__ == '__main__':
    unittest.main()



import os
import shutil
import tempfile
import unittest
import textwrap

import conda_build.config
from conda_build.metadata import MetaData

from conda_build_all.resolved_distribution import (ResolvedDistribution,
                                                   setup_vn_mtx_case)
from conda_build_all.tests.unit import RecipeCreatingUnit
from conda_build_all.tests.unit.dummy_index import DummyIndex, DummyPackage


class Test_BakedDistribution(RecipeCreatingUnit):
    # Tests cases where a recipe changes based on external
    # conditions, such as the definition of the PYTHON version.
    def test_py_version_selector(self):
        meta = self.write_meta("""
            package:
                name: recipe_which_depends_on_py_version
                version: 3  # [py3k]
                version: 2  # [not py3k]
            """)
        dist1 = ResolvedDistribution(meta, (('python', '27', ), ))
        dist2 = ResolvedDistribution(meta, (('python', '35', ), ))

        self.assertEqual(dist1.version(), u'2')
        self.assertEqual(dist2.version(), u'3')

    def test_py_version_selector_skip(self):
        meta = self.write_meta("""
            package:
                name: recipe_which_depends_on_py_version
            build:  # [py35]
                skip: True  # [py3k]
            """)
        dist1 = ResolvedDistribution(meta, (('python', '35', ), ))
        dist2 = ResolvedDistribution(meta, (('python', '34', ), ))

        self.assertEqual(dist1.skip(), True)
        self.assertEqual(dist2.skip(), False)


class Test_BakedDistribution_resolve_all(RecipeCreatingUnit):
    def test_py_xx_version(self):
        meta = self.write_meta("""
            package:
                name: recipe_which_depends_on_py_version
                version: 2
            requirements:
                build:
                 - python >=2.7
                 - numpy x.x
                run:
                 - python x.x
                 - numpy x.x
            """)
        self.index.add_pkg('python', '2.7.2')
        self.index.add_pkg('python', '2.6.2')
        self.index.add_pkg('python', '3.5.0')
        self.index.add_pkg('numpy', '1.8.0', depends=['python'])
        resolved = ResolvedDistribution.resolve_all(meta, self.index)
        ids = [dist.build_id() for dist in resolved]
        self.assertEqual(ids, ['np18py27_0', 'np18py35_0'])

    def test_skip_build(self):
        meta = self.write_meta("""
            package:
                name: recipe_which_depends_on_py_version
                version: 2
            build: # [py3k]
                skip: True  # [py3k]
            requirements:
                build:
                    - python
                run:
                    - python
            """)
        self.index.add_pkg('python', '2.7.2')
        self.index.add_pkg('python', '2.6.2')
        self.index.add_pkg('python', '3.5.0')
        resolved = ResolvedDistribution.resolve_all(meta, self.index)
        ids = [dist.build_id() for dist in resolved]
        self.assertEqual(ids, ['py26_0', 'py27_0'])

    def test_extra_conditions(self):
        meta = self.write_meta("""
                package:
                    name: test_recipe
                requirements:
                    build:
                        - python
                    run:
                        - python
                """)
        self.index.add_pkg('python', '2.7.2')
        self.index.add_pkg('python', '2.6.2')
        self.index.add_pkg('python', '3.5.0')
        resolved = ResolvedDistribution.resolve_all(meta, self.index,
                                       extra_conditions=['python 2.6.*|>=3'])
        ids = [dist.build_id() for dist in resolved]
        self.assertEqual(ids, ['py26_0', 'py35_0'])


class Test_setup_vn_mtx_case(unittest.TestCase):
    def test_perl_case(self):
        with setup_vn_mtx_case([('perl', '9.10.11.12'), ('numpy', '1.23'),
                                ('python', '2.7'), ('r', '4.5.6')]):
            self.assertEqual(conda_build.config.config.CONDA_PERL,
                             '9.10.11.12')
            self.assertEqual(conda_build.config.config.CONDA_NPY,
                             123)
            self.assertEqual(conda_build.config.config.CONDA_PY,
                             27)
            self.assertEqual(conda_build.config.config.CONDA_R,
                             '4.5.6')


if __name__ == '__main__':
    unittest.main()



from contextlib import contextmanager
import os
import shutil
import tempfile
import textwrap
import unittest

from conda_build.metadata import MetaData

from conda_build_all.resolved_distribution import ResolvedDistribution
from conda_build_all.builder import Builder
from conda_build_all.tests.unit.dummy_index import DummyIndex


sample_recipes = os.path.join(os.path.dirname(__file__), 'test_recipes')


class RecipeCreatingUnit(unittest.TestCase):
    def setUp(self):
        self.index = DummyIndex()
        self.directories_to_remove = []
        self.recipes_root_dir = self.tmp_dir(prefix='recipes')

    def tearDown(self):
        for directory in self.directories_to_remove:
            shutil.rmtree(directory)

    def tmp_dir(self, **mkdtemp_kwargs):
        tmp_dir = tempfile.mkdtemp(**mkdtemp_kwargs)
        self.directories_to_remove.append(tmp_dir)
        return tmp_dir

    def write_meta(self, recipe_dir_name, spec):
        recipe_dir = os.path.join(self.recipes_root_dir, recipe_dir_name)
        if not os.path.exists(recipe_dir):
            os.makedirs(recipe_dir)
        with open(os.path.join(recipe_dir, 'meta.yaml'), 'w') as fh:
            fh.write(textwrap.dedent(spec))
        return MetaData(recipe_dir)


class Test_build(RecipeCreatingUnit):
    def test_no_source(self):
        pkg1 = self.write_meta('pkg1', """
                    package:
                        name: pkg1
                        version: 1.0
                    """)
        pkg1_resolved = ResolvedDistribution(pkg1, (()))
        builder = Builder(None, None, None, None, None)
        r = builder.build(pkg1_resolved)
        self.assertTrue(os.path.exists(r))
        self.assertEqual(os.path.abspath(r), r)
        self.assertEqual(os.path.basename(r), 'pkg1-1.0-0.tar.bz2')

    def test_numpy_dep(self):
        pkg1 = self.write_meta('pkg1', """
                    package:
                        name: pkg1
                        version: 1.0
                    requirements:
                        build:
                            - python
                            - numpy x.x
                        run:
                            - python
                            - numpy x.x
                    """)
        pkg1_resolved = ResolvedDistribution(pkg1, (['python', '3.5'], ['numpy', '1.11']))
        builder = Builder(None, None, None, None, None)
        r = builder.build(pkg1_resolved)
        self.assertTrue(os.path.exists(r))
        self.assertEqual(os.path.abspath(r), r)
        self.assertEqual(os.path.basename(r), 'pkg1-1.0-np111py35_0.tar.bz2')


class Test__find_existing_built_dists(RecipeCreatingUnit):
    def make_channel(self, all_metas):
        for meta in all_metas:
            self.index.add_pkg_meta(meta)
        channel_root = tempfile.mkdtemp(prefix='temporary_channel_')
        # Line the directory up for removal when we're done with it.
        self.directories_to_remove.append(channel_root)
        
        self.index.write_to_channel(channel_root)
        return channel_root

    def setUp(self):
        super(Test__find_existing_built_dists, self).setUp()
        self.metas = {'a1': self.write_meta('a vn1', """
                    package:
                        name: a
                        version: 1.0
                    """),
                      'a2': self.write_meta('a vn2', """
                    package:
                        name: a
                        version: 2.0
                    """),
                      'a2_1': self.write_meta('a vn2 bld1', """
                    package:
                        name: a
                        version: 2.0
                    build:
                        number: 1
                    """),
                      'b2': self.write_meta('b vn2', """
                    package:
                        name: b
                        version: 2.0
                    requirements:
                        run:
                            - a 2.*
                    """)}

    def test_exist_on_channel(self):
        channel = self.make_channel(self.metas.values())
        channel_url = 'file://' + channel
        builder = Builder('.', [channel_url], [], [], [])
        expected_channel = '{}/{}'.format(channel_url, self.metas['a1'].info_index()['subdir'])
        existing = builder.find_existing_built_dists([self.metas['a1'], self.metas['a2']])
        dists = [(meta.dist(), locn) for meta, locn in existing]
        self.assertEqual(dists, [('a-1.0-0', expected_channel),
                                 ('a-2.0-0', expected_channel)])

    def test_full_version_exists_on_channel(self):
        # Only a vn2.0 build 1 is available, we want to assert that nothing is found for a1 and a2 build 0.
        channel = self.make_channel([self.metas['a2_1']])
        builder = Builder('.', ['file://' + channel], [], [], [])
        existing = builder.find_existing_built_dists([self.metas['a1'], self.metas['a2']])
        self.assertEqual([(meta.dist(), locn) for meta, locn in existing],
                         [('a-1.0-0', None), ('a-2.0-0', None)])


    def test_exists_in_directory(self):
        distribution_directory = tempfile.mkdtemp()
        # Line the directory up for removal when we're done with it.
        self.directories_to_remove.append(distribution_directory)

        with open(os.path.join(distribution_directory, self.metas['a1'].dist() + '.tar.bz2'), 'w') as fh:
            fh.write('placeholder')
        builder = Builder('.', [], [distribution_directory], [], [])
        existing = builder.find_existing_built_dists([self.metas['a1'], self.metas['a2']])
        dists = [(meta.dist(), locn) for meta, locn in existing]
        self.assertEqual(dists, [('a-1.0-0', distribution_directory), ('a-2.0-0', None)])


class Test_compute_build_distros(RecipeCreatingUnit):
    def test_added_to_index(self):
        metas = [self.write_meta('py2k', """
                    package:
                        name: python
                        version: 2.7.0
                    """),
                 self.write_meta('py33', """
                    package:
                        name: python
                        version: 3.3.0
                    """),
                 self.write_meta('py34', """
                    package:
                        name: python
                        version: 3.4.24
                    """),
                 self.write_meta('py35', """
                    package:
                        name: python
                        version: 3.5.2
                    build:
                        number: 1
                    """),
                 self.write_meta('np110', """
                    package:
                        name: numpy
                        version: 1.10
                    requirements:
                        build:
                            - python
                        run:
                            - python
                    """),
                self.write_meta('py_package', """
                    package:
                        name: my_py_package
                        version: 2.0
                    requirements:
                        build:
                            - python
                        run:
                            - python
                            - numpy
                    """)]
        builder = Builder(None, None, None, None, None)
        index = {}
        distributions = builder.compute_build_distros(index, metas)
        expected = ['python-2.7.0-0', 'python-3.3.0-0', 'python-3.4.24-0',
                    'python-3.5.2-1',
                    'numpy-1.10-py27_0', 'numpy-1.10-py34_0', 'numpy-1.10-py35_0',
                    'my_py_package-2.0-py27_0', 'my_py_package-2.0-py34_0',
                    'my_py_package-2.0-py35_0']
        self.assertEqual([meta.dist() for meta in distributions], expected)
        # Check that we didn't change the index.
        self.assertEqual(index, {}) 


if __name__ == '__main__':
    unittest.main()
